/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (C) 2024 KylinSoft  Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#ifndef BOOTINTERFACE_H
#define BOOTINTERFACE_H

#include <ukui-service-interface/serviceobject.h>
#include <QObject>
#include <QtDBus>
#include <QDBusContext>
#include <QDBusConnectionInterface>
#include "grubtask.h"

#define UKCC_SCHAME "org.ukui.control-center"
#define SYSTEMS_SAME "systems-same"

class BootInterface : public UKUI::ServiceObject
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.ukui.ukcc.session.Boot")

public:
    BootInterface();
    ~BootInterface();

Q_SIGNALS:
    void finished(int type, bool reply);

public Q_SLOTS:
    void addTask(QString pwd, bool isOpen, int type);
    void addTask(int order, int type);
    bool getVerifyStatus();
    int getBootDelay();
    int getGrubDefault();

private:
    QThreadPool *m_pThread;
    QDBusUnixFileDescriptor m_inhibitFileDescriptor;
    QVector<GrubTask*> runlist;
    QVector<GrubTask*> waitlist;
    bool m_needVerify;
    int m_bootDelay;
    int m_grubDefault;

private:
    bool inhibit(QString what, QString who, QString why, QString mode);
    void uninhibit();
    bool isSameSystems();

private slots:
    void taskFinishedSlot(bool reply);
};

#endif // BOOTINTERFACE_H
